@@ -1,20 +1,25 @@
+{-# LANGUAGE RecursiveDo #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 {-----------------------------------------------------------------------------
     reactive-banana
 ------------------------------------------------------------------------------}
-{-# LANGUAGE RecursiveDo, ScopedTypeVariables #-}
 module Reactive.Banana.Prim.Mid.Combinators where
 
 import Control.Monad
+    ( join )
 import Control.Monad.IO.Class
+    ( liftIO )
 
-import Reactive.Banana.Prim.Low.Plumbing
+import Reactive.Banana.Prim.Mid.Plumbing
     ( newPulse, newLatch, cachedLatch
     , dependOn, keepAlive, changeParent
     , getValueL
     , readPulseP, readLatchP, readLatchFutureP, liftBuildP,
     )
-import qualified Reactive.Banana.Prim.Low.Plumbing (pureL)
-import           Reactive.Banana.Prim.Low.Types    (Latch, Future, Pulse, Build, EvalP)
+import qualified Reactive.Banana.Prim.Mid.Plumbing
+    ( pureL )
+import Reactive.Banana.Prim.Mid.Types
+    ( Latch, Future, Pulse, Build, EvalP )
 
 debug :: String -> a -> a
 -- debug s = trace s
@@ -85,7 +90,7 @@ applyP f x = do
     return p
 
 pureL :: a -> Latch a
-pureL = Reactive.Banana.Prim.Low.Plumbing.pureL
+pureL = Reactive.Banana.Prim.Mid.Plumbing.pureL
 
 -- specialization of   mapL f = applyL (pureL f)
 mapL :: (a -> b) -> Latch a -> Latch b
@@ -132,22 +137,25 @@ executeP p1 b = do
     eval Nothing  = return Nothing
 
 switchP :: Pulse a -> Pulse (Pulse a) -> Build (Pulse a)
-switchP p pp = mdo
+switchP p pp = do
+    -- track the latest Pulse in a Latch
     lp <- stepperL p pp
-    let
-        -- switch to a new parent
+
+    -- fetch the latest Pulse value
+    pout <- newPulse "switchP_out" (readPulseP =<< readLatchP lp)
+
+    let -- switch the Pulse `pout` to a new parent,
+        -- keeping track of the new dependencies.
         switch = do
             mnew <- readPulseP pp
             case mnew of
-                Nothing  -> return ()
-                Just new -> liftBuildP $ p2 `changeParent` new
-            return Nothing
-        -- fetch value from old parent
-        eval = readPulseP =<< readLatchP lp
+                Nothing  -> pure ()
+                Just new -> liftBuildP $ pout `changeParent` new
+            pure Nothing
 
-    p1 <- newPulse "switchP_in" switch :: Build (Pulse ())
-    p1 `dependOn` pp
-    p2 <- newPulse "switchP_out" eval
-    p2 `dependOn` p
-    p2 `keepAlive` p1
-    return p2
+    pin <- newPulse "switchP_in" switch :: Build (Pulse ())
+    pin  `dependOn` pp
+    
+    pout `dependOn` p       -- initial dependency
+    pout `keepAlive` pin    -- keep switches happening
+    pure pout
